; tapserv.asm
; C64 side server for TAP transfers
; Version 1.3
; (C) 2000 Markus Brenner


; ATN OUT: Bit #3 - indicate sense ('PLAY PRESSED')
; CLK IN:  Bit #6 - data pulse ('receive mode')
; CLK OUT: Bit #4 - data pulse ('send    mode')
; DATA IN: Bit #7 - tapserv mode:
;          0 = send    mode
;          1 = receive mode

DATAIN  = %10000000
CLKIN   = %01000000
DATAOUT = %00100000
CLKOUT  = %00010000
ATNOUT  = %00001000

CASOUT  = %00001000
CASSENSE= %00010000

; Receive Mode: DATAIN = TRUE
SENDMODE= %10000000
RECVMODE= 0


* = $0801

.INCLUDE SYSTEM.ASM			; KERNAL functions

                ; BASIC header "2000 SYS 2062"

		.byte $26,$08,$d0,$07,$9e,$20,$32,$30
                .byte $36,$32,$00,$00,$00

		JSR  init_screen


		LDA  $01
                AND  #CASSENSE
                EOR  #CASSENSE		; Sense, 0=play button down
		STA  sense_out		; init _sense_out value
		JSR  screen_color	; set screen color to _sense_out

		SEI
		JSR  copy_kernal	; copy kernal, enable KERNAL RAM
		LDA  $01
		AND  #$d7		; Motor = ON (Bit 5 = 0)
					; Cassette data = 0
                STA  $fc		; cassette output byte (0)
		ORA  #CASOUT		; cassette data = 1
		STA  $fd		; cassette output byte (1)
                JSR  init_CIA		; set CIA2 to output, clear CLK,DATA OUT
                JSR  init_ints

 
                LDA  #$ff
                STA  serv_mode		; make sure, main_loop sets server mode
                STA  clk_in		; clk_in not initialized yet


					; check server mode
main_loop       LDA  CIA2		; check if server mode changes
                AND  #DATAIN
                CMP  serv_mode
		BEQ  main_loopJ2
		STA  serv_mode		; change mode
		JSR  pr_serv_mode	; print new server mode
                JSR  init_CIA		; set CIA2 to output, clear CLK,DATA OUT
		JSR  set_ints		; disable/allow interrupts
		JMP  main_loop


main_loopJ2     LDA  $01
                AND  #CASSENSE		; Sense Bit, 0=play button down
                EOR  #CASSENSE
		CMP  sense_out
		BEQ  main_loopJ3

		SEI
                STA  sense_out		; sense = Bit #4
		LSR
		STA  $fb		; Shift into ATN OUT bit position
		LDA  cia_out
		AND  #$f7		; clear ATN OUT bit
		ORA  $fb
		STA  cia_out		; set new 'sense' state in CIA byte
		STA  CIA2
		JSR  set_ints		; disable/allow interrupts
		JSR  screen_color

		LDA  sense_out		; turn screen off if playing
		AND  #CASSENSE
		CMP  #CASSENSE
		BEQ  screen_off	; PLAY key pressed ->

screen_on       LDA  $d011
		ORA  #$10
		STA  $d011
                JMP  main_loop

screen_off      LDA  $d011
		AND  #$ef
		STA  $d011
                JMP  main_loop

main_loopJ3     LDA  serv_mode
                CMP  #RECVMODE
		BEQ  receive_loop
                JMP  main_loop		; if in server mode, continue main loop
					; (pulses are handled in interrupt)

receive_loop	LDA  CIA2
		CMP  last_cia		; wait for change in CIA
		BEQ  receive_loop
		PHA
		AND  #CLKIN
                BNE  receiveJ1
		LDA  $fc		; write "0" to tape
                STA  $01
		JMP  receiveJ2
receiveJ1       LDA  $fd		; write "1" to tape
                STA  $01
receiveJ2       JSR  border_cycle
		PLA			; (CIA state)
		STA  last_cia
		AND  #SENDMODE
		BNE  receiveEnd
		LDA  $01
                AND  #CASSENSE		; Sense Bit, 0=play button down
		BEQ  receive_loop
receiveEnd	JMP  main_loop


;receive_loop    LDA  #CLKIN
;                AND  CIA2
;                BNE  receiveJ1
;		LDA  $fc		; write "0" to tape
;                STA  $01
;                LDA  #CLKIN
;                JMP  receiveJ2
;receiveJ1       LDA  $fd		; write "1" to tape
;                STA  $01
;                LDA  #$00
;receiveJ2       CMP  clk_in
;		BEQ  receiveEnd
;                STA  clk_in
;                JSR  border_cycle
;receiveEnd      JMP  main_loop


screen_color	LDA  sense_out
		LSR
		LSR
		LSR
		LSR
		STA  $d021
		RTS



my_irq          SEI
                PHA
                LDA  $dc0d		; CIA interrupt control

                LDA  cia_out		; toggle output byte
                EOR  #CLKOUT
                STA  cia_out 
                STA  CIA2

		JSR  border_cycle

                PLA
                RTI

my_nmi		RTI			; direct return from NMI

serv_mode	.byte $00
cia_out 	.byte $00
clk_in		.byte $00
sense_out	.byte $00
cycle		.byte $00
last_cia	.byte $00



copy_kernal     LDY  #$00
ckL             LDA  $a000,Y
		STA  $a000,Y
		INY
		BNE  ckL
		INC  ckL+2
		INC  ckL+5
		LDA  ckL+2
		BNE  ckL
		
                LDA  $01
                AND  #%11011111		; turn datasette power on
                AND  #%11111101		; Kernal RAM !
                STA  $01
                RTS


init_CIA	LDA  CIA2
                AND  #%11110111		; clear ATN out <PLAY>
;		ORA  #DATAOUT		; (for reading set CLK/DATA OUT)
;		ORA  #CLKOUT
                AND  #%11001111		; clear CLK/DATA OUT
                STA  $fb
		LDA  sense_out
		LSR
		ORA  $fb		; Shift into ATN OUT bit position
                STA  cia_out		; init output value
		STA  CIA2

                RTS


init_ints       LDA  #%00011111		; CLEAR all IRQ interrupts
                STA  $dc0d		; CIA Interrupt control register

                LDA  #%10010000		; SET CASS IRQ interrupt
                STA  $dc0d		; 

                LDA  #<my_irq		; set new irq vector
                STA  $fffe
                LDA  #>my_irq
                STA  $ffff

                LDA  #<my_nmi		; set new nmi vector
                STA  $fffa
                LDA  #>my_nmi
                STA  $fffb

                RTS


init_screen	JSR  $e544		; clear screen
		JSR  $e566		; home

		LDA  #$0f		; "GREY 3"
		STA  $0286		; COLOR

		LDA  #<screen_text
		LDY  #>screen_text
		JSR  $ab1e		; print string from A/Y
		RTS


pr_serv_mode    JSR  $e9ff		; clear screen
		CLC
		LDX  #$0b		; row     1 (0-24)
		LDY  #$00		; column 10 (0-39)
		JSR  PLOT
		LDA  serv_mode
                CMP  #RECVMODE
		BEQ  pr_receive
		LDA  #<send_text
		LDY  #>send_text
		JMP  pr_print
pr_receive	LDA  #<receive_text
		LDY  #>receive_text
pr_print	JSR  $ab1e		; print string from A/Y
		RTS


set_ints	LDA  serv_mode
                CMP  #RECVMODE
		BEQ  ints_receive
		CLI			; allow   interrupts in send mode
		RTS
ints_receive	SEI			; disable interrupts in receive mode
		RTS


border_cycle	CLC
                LDA  cycle		; perform color cycle
                ADC  #$01
                AND  #$0f
                STA  cycle
                STA  $d020
		RTS


screen_text	.byte $0d
		.text "          **** TAPSERV 1.3 ****"
		.byte $0d,$0d
		.text "         (C) 2000 MARKUS BRENNER"
		.byte $00

send_text	.text "                SEND MODE "
		.byte $00

receive_text	.text "              RECEIVE MODE"
		.byte $00
